/**
 * CMI : Cluster Method Invocation
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: cmi@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 * --------------------------------------------------------------------------
 * $Id: StubOrProxyHelper.java 2223 2008-11-13 17:08:31Z loris $
 * --------------------------------------------------------------------------
 */

/**
 *
 */
package org.ow2.cmi.rpc;

import java.rmi.Remote;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.ow2.cmi.controller.common.ClusterViewManager;
import org.ow2.cmi.controller.server.ProtocolNotFoundException;
import org.ow2.cmi.reference.CMIReference;
import org.ow2.cmi.reference.CMIReferenceable;
import org.ow2.cmi.reference.CMIReferenceableWrapper;
import org.ow2.cmi.reference.ObjectNotFoundException;
import org.ow2.cmi.reference.ServerRef;
import org.ow2.cmi.reference.serialization.RemoteIIOPCMIReferenceableWrapper;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Helper to retrieve RMI stubs and proxies.
 * @author Loris Bouzonnet
 */
public final class StubOrProxyHelper {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(StubOrProxyHelper.class);

    /**
     * No public constructor.
     */
    private StubOrProxyHelper() {
        // Do nothing
    }

    /**
     * @param cmiReference
     * @return
     * @throws ProtocolNotFoundException
     * @throws ObjectNotFoundException
     * @throws NamingException
     * @throws ClassCastException
     */
    public static CMIReferenceable<?> download(final ClusterViewManager clusterViewManager, final CMIReference cmiReference)
    throws ProtocolNotFoundException, ObjectNotFoundException, ClassCastException, NamingException {
        ServerRef serverRef = cmiReference.getServerRef();
        String protocol = serverRef.getProtocol();
        // Get a context to perform a lookup
        Hashtable<String, Object> env = new Hashtable<String, Object>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, clusterViewManager.getInitialContextFactoryName(protocol));
        env.put(Context.PROVIDER_URL, serverRef.getProviderURL());
        Context chosenContext = new InitialContext(env);

        // Get a JNDI name to perform a lookup
        String bindName = cmiReference.getObjectName();
        Class<?> interfaceClass = clusterViewManager.getInterface(bindName);

        // Lookup...
        logger.debug("Lookup {0} on {1}", bindName, serverRef.getProviderURL());
        Object object;
        try {
            object = PortableRemoteObject.narrow(chosenContext.lookup(bindName), interfaceClass);
        } finally {
            try {
                chosenContext.close();
            } catch (NamingException e) {
                logger.debug("Unable to close the context", e);
            }
        }

        // Construct a CMIReferenceable with the given CMIReference and the retrieved object
        if(protocol.equals("iiop") && object instanceof Remote) {
            return new RemoteIIOPCMIReferenceableWrapper<Remote>(cmiReference, (Remote) object);
        } else {
            return new CMIReferenceableWrapper<Object>(cmiReference, object);
        }

    }

}
